package com.jiuqi.util.chinese;

import java.io.Serializable;

class ChineseDigit implements Comparable, Cloneable, Serializable
{
    private static final char[] SMALL_DIGITS;
    private static final char[] BIG_DIGITS;
    private static char bufferedChar;
    private static int bufferedValue;
    private int digit;
    
    static {
        SMALL_DIGITS = new char[] { '○', '一', '二', '三', '四', '五', '六', '七', '八', '九' };
        BIG_DIGITS = new char[] { '零', '壹', '贰', '叁', '肆', '伍', '陆', '柒', '捌', '玖' };
        ChineseDigit.bufferedChar = '\0';
        ChineseDigit.bufferedValue = -1;
    }
    
    public ChineseDigit() {
        this.digit = 0;
    }
    
    public ChineseDigit(final int dig) {
        if (dig >= 0 && dig <= 9) {
            this.digit = dig;
            return;
        }
        throw new IllegalArgumentException("只能从0-9的阿拉伯数字来建立对象！");
    }
    
    public ChineseDigit(final char digChar) throws NotValidChineseDigitException {
        final int value = digChar - '0';
        if (value >= 0 && value <= 9) {
            this.digit = value;
        }
        else {
            this.digit = valueOf(digChar);
        }
    }
    
    public int intValue() {
        return this.digit;
    }
    
    public char chineseValue(final boolean useBigChar) {
        return toChineseChar(this.digit, useBigChar);
    }
    
    public static int valueOf(final char chDig) throws NotValidChineseDigitException {
        if (ChineseDigit.bufferedChar == chDig) {
            return ChineseDigit.bufferedValue;
        }
        int position = -1;
        for (int i = 0; i < ChineseDigit.SMALL_DIGITS.length; ++i) {
            if (chDig == ChineseDigit.SMALL_DIGITS[i]) {
                position = i;
                break;
            }
        }
        if (position != -1) {
            ChineseDigit.bufferedChar = chDig;
            return ChineseDigit.bufferedValue = position;
        }
        for (int i = 0; i < ChineseDigit.BIG_DIGITS.length; ++i) {
            if (chDig == ChineseDigit.BIG_DIGITS[i]) {
                position = i;
                break;
            }
        }
        if (position != -1) {
            ChineseDigit.bufferedChar = chDig;
            return ChineseDigit.bufferedValue = position;
        }
        throw new NotValidChineseDigitException("字符\"" + chDig + "\"不是一个中文的数字字符！");
    }
    
    public static char toChineseChar(final int value, final boolean useBigChar) {
        if (value < 0 || value > 9) {
            throw new IllegalArgumentException("整型参数值\"" + value + "\"必须在0-9之间！");
        }
        if (!useBigChar) {
            return ChineseDigit.SMALL_DIGITS[value];
        }
        return ChineseDigit.BIG_DIGITS[value];
    }
    
    public static boolean isValidChineseDigit(final char value) {
        try {
            valueOf(value);
            return true;
        }
        catch (final NotValidChineseDigitException ex) {
            return false;
        }
    }
    
    public Object clone() {
        return new ChineseDigit(this.digit);
    }
    
    @Override
    public boolean equals(final Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof ChineseDigit)) {
            return false;
        }
        final ChineseDigit value = (ChineseDigit)o;
        return this.digit == value.digit;
    }
    
    @Override
    public int hashCode() {
        int result = super.hashCode();
        final int hc1 = this.digit;
        result = 37 * result + hc1;
        return result;
    }
    
    @Override
    public String toString() {
        return String.valueOf(this.chineseValue(false));
    }
    
    public int compareTo(final Object o) {
        final ChineseDigit value = (ChineseDigit)o;
        return this.digit - value.digit;
    }
}
